package parser

import "regexp"

type deviceTableElement struct {
	name     string
	reString string
	re       *regexp.Regexp
}

var deviceTable = []deviceTableElement{
	{"AMBA PL011 UART Device", "Serial: AMBA PL011 UART driver", nil},
	{"BCM2835 Mailbox", "bcm2835-mbox[[:print:]]+mailbox enabled", nil},
	{"BCM2835 DMA", "bcm2835-dma[[:print:]]+DMA legacy API manager", nil},
	{"RaspberryPi Firmware", "raspberrypi-firmware soc:firmware: Attached to firmware", nil},
	{"BCM2708FB", "BCM2708FB: allocated DMA memory", nil},
	{"BCM2835 RNG", "bcm2835-rng[[:print:]]+hwrng registered", nil},
	{"Videocore Memory", "vc-mem: phys_addr:", nil},
	{"Videocore Shared Memory", "vc-sm: Videocore shared memory driver", nil},
	{"Microchip LAN78XX Based USB Ethernet Adapter", "new interface driver lan78xx", nil},
	{"SMSC LAN95XX Based USB 2.0 10/100 Ethernet Device", "new interface driver smsc95xx", nil},
	{"DesignWare OTG", "dwc_otg: version ", nil},
	{"USB Mass Storage", "new interface driver usb-storage", nil},
	{"BCM2835 Watchdog Timer", "bcm2835-wdt[[:print:]]+Broadcom BCM2835 watchdog timer", nil},
	{"BCM2835 CPU Frequency Scaler", "bcm2835-cpufreq: min=", nil},
	{"Secure Digital Host Controller Interface", "sdhci: Secure Digital Host Controller Interface driver", nil},
	{"8250/16550 Serial Device", "Serial: 8250/16550 driver", nil},
	{"SDHC Card", "new high speed SDHC card", nil},
	{"SDIO Card", "new high speed SDIO card", nil},
	{"S3C Power Management", "S3C Power Management, Copyright 2004 Simtec Electronics", nil},
	{"SCSI subsystem", "SCSI subsystem initialized", nil},
	{"ARM Mali", "Mali: Mali device driver loaded", nil},
	{"Generic vibrator", "vibrator_init: vibrator", nil},
	{"Broadcom Dongle Host Driver", "dhd_module_init in", nil},
	{"Intel® Performance Counter Monitor", "intel_pmu_driver [[:print:]]+: after pmu initialization", nil},
	{"Generic Wi-Fi", "found wifi platform device wlan", nil},
	{"Broadcom SDIO", "bcmsdh_sdmmc: bcmsdh_sdmmc_probe Enter", nil},
	{"s3c64xx SPI", "s3c64xx-spi 13920000.spi: spi bus clock parent not specified, using clock at index 0 as parent", nil},
	{"NLS-HR22", "hid-generic [0-9]+:1EAB:1A03.[0-9]+: input,hidraw[0-9]+: USB HID v1.00 Keyboard", nil},
	{"CP210X", "cp210x [0-9]+-[0-9]+.[0-9]+:[0-9]+.[0-9]+: cp210x converter detected", nil},
	{"NetWinder Floating Point Emulator", "NetWinder Floating Point Emulator", nil},
	{"Block layer SCSI generic", "Block layer SCSI generic \\(bsg\\) driver", nil},
	{"SCSI Media Changer", "SCSI Media Changer driver", nil},
	{"Nuvoton NUC970 EHCI Host Controller", "Nuvoton NUC970 EHCI Host Controller", nil},
	{"Nuvoton NUC970 OHCI Host Controller", "Nuvoton NUC970 OHCI Host Controller", nil},
	{"Wireless RNDIS USB", "new interface driver rndis_wlan", nil},
	{"ASIX AX88xxx Based USB 2.0 Ethernet Adapters", "new interface driver asix", nil},
	{"ASIX AX88179/178A USB 3.0/2.0 to Gigabit Ethernet", "new interface driver ax88179_178a", nil},
	{"CDC Ethernet", "new interface driver cdc_ether", nil},
	{"CDC EEM", "new interface driver cdc_eem", nil},
	{"NetChip 1080 based cables (Laplink, ...)", "new interface driver net1080", nil},
	{"Host for RNDIS and ActiveSync devices", "new interface driver rndis_host", nil},
	{"Simple USB Network Links (CDC Ethernet subset)", "new interface driver cdc_subset", nil},
	{"Sharp Zaurus (stock ROMs) and compatible", "new interface driver zaurus", nil},
	{"CDC NCM", "new interface driver cdc_ncm", nil},
	{"CDC MBIM", "new interface driver cdc_mbim", nil},
	{"I2C EEPROM", "at24 .* EEPROM", nil},
}

func init() {
	for i, e := range deviceTable {
		deviceTable[i].re = regexp.MustCompile(e.reString)
	}
}
